// Copyright (c) 1999-2013  Pavel Rychly, Milos Jakubicek

#ifndef FINLIB_COMPSTREAM_HH
#define FINLIB_COMPSTREAM_HH

#include <finlib/fstream.hh>
#include <finlib/bitio.hh>

//-------------------- GolombPosStream<bitIterator> --------------------

template <class bitIterator>
class GolombPosStream: public FastStream {
protected:
    Position b_gol;
    bitIterator bits;
    read_bits <bitIterator&> rb;
    Position finval;
    NumOfPos rest;
    Position current;
    void read_next () {
        if (rest > 0) {
            rest--;
            current += rb.golomb (b_gol);
        } else
            current = finval;
    }

public:
    GolombPosStream (bitIterator &bi, NumOfPos count, Position b, Position fin)
        : b_gol (b), rest (count), finval (fin), current (-1), 
          bits (bi), rb (bits) {read_next();}
    virtual Position peek () {return current;}
    virtual Position next () {
        Position ret = current;
        read_next();
        return ret;
    }
    virtual Position find (Position pos) {
        while (current < pos && current < finval)
            read_next();
        return current;
    }
    virtual NumOfPos rest_min () {return rest + (current!=finval);}
    virtual NumOfPos rest_max () {return rest + (current!=finval);}
    virtual Position final () {return finval;}
};


//-------------------- DeltaPosStream<bitIterator> --------------------

template <class bitIterator>
class DeltaPosStream: public FastStream {
protected:
    bitIterator bits;
    read_bits <bitIterator&, typename std::iterator_traits
              <bitIterator>::value_type, Position> rb;
    Position finval;
    NumOfPos rest;
    Position current;
    void read_next () {
        if (rest > 0) {
            rest--;
            current += rb.delta();
        } else
            current = finval;
    }

public:
    DeltaPosStream (bitIterator bi, NumOfPos count, Position fin, int skipbits=0)
        : bits (bi), rb (bits, skipbits), finval (fin), rest (count), 
        current (-1)
        {read_next();}
    virtual Position peek () {return current;}
    virtual Position next () {
        Position ret = current;
        read_next();
        return ret;
    }
    virtual Position find (Position pos) {
        while (current < pos && current < finval)
            read_next();
        return current;
    }
    virtual NumOfPos rest_min () {return rest + (current!=finval);}
    virtual NumOfPos rest_max () {return rest + (current!=finval);}
    virtual Position final () {return finval;}
    bool operator == (const DeltaPosStream &x) const {return rb == x.rb;}
};


#endif

// vim: ts=4 sw=4 sta et sts=4 si cindent tw=80:
