// Copyright (c) 1999-2014  Pavel Rychly, Milos Jakubicek

#ifndef FINLIB_REVIDX_HH
#define FINLIB_REVIDX_HH

#include <finlib/config.hh>
#include <finlib/compstream.hh>
#include <finlib/binfile.hh>
#include <finlib/bititer.hh>
#include <finlib/fsop.hh>
#include <finlib/fromtof.hh>
#include <string>
#include <vector>
#include <iterator>

#if defined __GNUC__ && __GNUC__ >= 3
#include <ext/hash_map>
#else
#include <hash_map>
#endif

#if defined __GNUC__ && (__GNUC__ == 3 && __GNUC_MINOR__ >= 1 || __GNUC__ >= 4)
#define HASH_NAMESPACE __gnu_cxx::
#else
#define HASH_NAMESPACE std::
#endif

template <class RevFileClass, class IdxFileClass=MapBinFile<uint32_t> >
class delta_revidx
{
protected:
    RevFileClass crevf;
    IdxFileClass crdxf;
    IdxFileClass cntf;
    const Position maxpos_val;
    int maxid_val;
    int alignmult;
    HASH_NAMESPACE hash_map<int, NumOfPos> cnt_ovf;
    static const size_t isize = sizeof (typename std::iterator_traits
                         <typename RevFileClass::const_iterator>::value_type);
public:
    delta_revidx (const std::string &filename, Position mpos=maxPosition,
                  int align=1) 
        : crevf (filename + ".rev"), crdxf (filename + ".rev.idx"), 
          cntf (filename + ".rev.cnt"), maxpos_val (mpos), maxid_val (0),
          alignmult (align) {
        if (crdxf[0] > 0) {
            // read alignmult
             DeltaPosStream<typename RevFileClass::const_iterator> 
                 dps (crevf.at (6 / isize), 1, 1024, 6 % isize * 8);
             alignmult = dps.next();
        }
        try {
            FromFile<int64_t> cnt64(filename + ".rev.cnt64");
            while (!cnt64)
                cnt_ovf[cnt64.get()] = cnt64.get();
        } catch (FileAccessError &e) {
            if (e.err != ENOENT)
                throw;
        }
    }
    //~delta_revidx ();
    FastStream *id2poss (int id) {
        NumOfPos c;
        if (id >= 0 && (c = count(id)) > 0) {
            const off_t seek = off_t(crdxf [id]) * off_t(alignmult);
            FastStream *fs =
                new DeltaPosStream<typename RevFileClass::const_iterator>
                (crevf.at (seek / isize), c, maxpos_val, seek % isize * 8);
            if (c < RevFileClass::buffsize)
                fs = new Fast2Mem <Position> (fs, c);
            return fs;
        } else
            return new EmptyStream();
    }
    NumOfPos count (int id) {
        HASH_NAMESPACE hash_map<int, NumOfPos>::const_iterator it = cnt_ovf.find(id);
        if (it != cnt_ovf.end())
            return it->second;
        return cntf [id];
    }
    int maxid () {
        if (!maxid_val)
            maxid_val = cntf.size();
        return maxid_val;
    }
    off_t data_size () const {return crevf.size();}
    Position maxpos() const {return maxpos_val;}
};


typedef delta_revidx<MapBinFile<uint64_t> > map_delta_revidx;
typedef delta_revidx<BinCachedFile<uint64_t> > file_delta_revidx;
typedef delta_revidx<BinCachedFile<uint64_t>,
                     BinFile<uint32_t> > file_file_delta_revidx;
    

#endif

// vim: ts=4 sw=4 sta et sts=4 si cindent tw=80:
